'use strict';

var _get = require('babel-runtime/helpers/get')['default'];

var _inherits = require('babel-runtime/helpers/inherits')['default'];

var _createClass = require('babel-runtime/helpers/create-class')['default'];

var _classCallCheck = require('babel-runtime/helpers/class-call-check')['default'];

var _Object$keys = require('babel-runtime/core-js/object/keys')['default'];

var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default')['default'];

var _interopRequireWildcard = require('babel-runtime/helpers/interop-require-wildcard')['default'];

Object.defineProperty(exports, '__esModule', {
    value: true
});

var _events = require('events');

var _events2 = _interopRequireDefault(_events);

var _allureJsCommonsWorkflo = require('allure-js-commons-workflo');

var _allureJsCommonsWorkflo2 = _interopRequireDefault(_allureJsCommonsWorkflo);

var _allureJsCommonsWorkfloBeansStep = require('allure-js-commons-workflo/beans/step');

var _allureJsCommonsWorkfloBeansStep2 = _interopRequireDefault(_allureJsCommonsWorkfloBeansStep);

var _fs = require('fs');

var fs = _interopRequireWildcard(_fs);

function isEmpty(object) {
    return !object || _Object$keys(object).length === 0;
}

var LOGGING_HOOKS = ['"before all" hook', '"after all" hook'];

var debug = false;
var debugSeleniumCommand = false;

function logger() {
    if (debug) {
        console.log(arguments);
    }
}
function error() {
    console.error(arguments);
}

/**
 * Initialize a new `Allure` test reporter.
 *
 * @param {Runner} runner
 * @api public
 */

var AllureReporter = (function (_events$EventEmitter) {
    _inherits(AllureReporter, _events$EventEmitter);

    function AllureReporter(baseReporter, config) {
        var _this = this;

        var options = arguments.length <= 2 || arguments[2] === undefined ? {} : arguments[2];

        _classCallCheck(this, AllureReporter);

        _get(Object.getPrototypeOf(AllureReporter.prototype), 'constructor', this).call(this);

        this.baseReporter = baseReporter;
        this.config = config;
        this.options = options;
        if (options) {
            debug = options.debug;
            debugSeleniumCommand = options.debugSeleniumCommand;
        }
        this.allures = {};
        var epilogue = this.baseReporter.epilogue;

        this.on('end', function () {
            epilogue.call(baseReporter);
        });

        this.on('suite:start', function (suite) {
            var allure = _this.getAllure(suite.cid);
            var currentSuite = allure.getCurrentSuite();
            var prefix = currentSuite ? currentSuite.name + ' ' : '';

            process.workflo.currentCid = suite.cid;

            allure.startSuite(prefix + suite.title);
        });

        this.on('suite:end', function (suite) {
            _this.getAllure(suite.cid).endSuite();
        });

        this.on('test:start', function (test) {
            var allure = _this.getAllure(test.cid);
            allure.startCase(test.title);

            var currentTest = allure.getCurrentTest();

            if (test.testcase) {
                currentTest.addParameter('argument', 'Testcase', test.testcase);
            } else if (test.spec) {
                currentTest.addParameter('argument', 'Spec', test.spec);
            }

            if (test.metadata) {
                for (var key in test.metadata) {
                    var meta = {
                        event: 'test:meta',
                        cid: test.cid
                    };
                    meta[key] = test.metadata[key];

                    _this.handleMetadata(meta);
                }
            }

            // add spec ids

            // these will be added on dashboard page
            currentTest.addParameter('environment-variable', 'BROWSER', test.runner[test.cid].browserName);
            currentTest.addParameter('environment-variable', 'BASE URL', _this.config.baseUrl);
            currentTest.addParameter('environment-variable', 'WEBDRIVER HOST', _this.config.host);
            currentTest.addParameter('environment-variable', 'WEBDRIVER PORT', _this.config.port);
            currentTest.addParameter('environment-variable', 'SELENIUM CAPABILITIES', JSON.stringify(test.runner[test.cid]));
        });

        this.on('test:pass', function (test) {
            var allure = _this.getAllure(test.cid);

            _this.addArguments(test);
            allure.endCase('passed');
        });

        this.on('test:fail', function (test) {
            var allure = _this.getAllure(test.cid);

            if (!allure.getCurrentTest()) {
                allure.startCase(test.title);
            } else {
                allure.getCurrentTest().name = test.title;
            }

            while (allure.getCurrentSuite().currentStep instanceof _allureJsCommonsWorkfloBeansStep2['default']) {
                allure.endStep('failed');
            }

            _this.addArguments(test);
            allure.endCase('failed', {
                message: '\n' + test.err.message,
                stack: '\n' + test.err.stack
            });
        });

        this.on('test:broken', function (test) {
            var allure = _this.getAllure(test.cid);

            if (!allure.getCurrentTest()) {
                allure.startCase(test.title);
            } else {
                allure.getCurrentTest().name = test.title;
            }

            while (allure.getCurrentSuite().currentStep instanceof _allureJsCommonsWorkfloBeansStep2['default']) {
                allure.endStep('broken');
            }

            _this.addArguments(test);
            allure.endCase('broken', {
                message: '\n' + test.err.message,
                stack: '\n' + test.err.stack
            });
        });

        this.on('test:pending', function (test) {
            var allure = _this.getAllure(test.cid);

            _this.addArguments(test);

            allure.endCase('pending', { message: 'Test ignored', stack: '' });
        });

        this.on('test:unverified', function (test) {
            var allure = _this.getAllure(test.cid);

            if (!allure.getCurrentTest()) {
                allure.startCase(test.title);
            } else {
                allure.getCurrentTest().name = test.title;
            }

            _this.addArguments(test);
            allure.endCase('unknown', {
                message: '\n' + test.err.message
            });
        });

        this.on('runner:command', function (command) {
            var allure = _this.getAllure(command.cid);

            if (!_this.isAnyTestRunning(allure)) {
                return;
            }

            if (debugSeleniumCommand) {
                allure.startStep(command.method + ' ' + command.uri.path);

                if (!isEmpty(command.data)) {
                    _this.dumpJSON(allure, 'Request', command.data);
                }
            }
        });

        this.on('runner:result', function (command) {
            var allure = _this.getAllure(command.cid);
            var status = 'passed';

            if (!_this.isAnyTestRunning(allure)) {
                return;
            }

            if (debugSeleniumCommand) {
                if (command.requestOptions.uri.path.match(/\/wd\/hub\/session\/[^/]*\/screenshot/)) {
                    allure.addAttachment('Screenshot', new Buffer(command.body.value, 'base64'));
                } else {
                    if (command.body.screenshot) {
                        allure.addAttachment('Screenshot', new Buffer(command.body.screenshot, 'base64'));
                        delete command.body.screenshot;
                    }
                    if (command.body.type && command.body.type === 'RuntimeError') {
                        status = 'broken';
                    }

                    _this.dumpJSON(allure, 'Response', command.body);
                }
            }

            allure.endStep(status);
        });

        /*
           meta : {
            (mandatory) cid : ...,
            (mandatory) event : 'test:meta'
            (optional) description : string
            (optional) feature : string | array
            (optional) story : string | array
            (optional) issue : string | array
            (optional) severity : [ 'blocker','critical','normal','minor','trivial' ]
            (optional) argument : {name: value, name2: value }
            (optional) environment : {name: value, name2: value }
           }
        */
        this.on('test:meta', function (meta) {
            this.handleMetadata(meta);
        });

        this.on('step:start', function (step) {
            var allure = this.getAllure(step.cid);
            logger('step:start', step);

            if (!this.isAnyTestRunning(allure)) {
                error('ERROR', 'cannot start step because no test is running', step);
                return;
            }
            allure.startStep(step.title != null ? step.title : 'No name defined');

            if (step.arg) {
                this.dumpJSON(allure, 'Step Arg', step.arg);
            }
        });

        this.on('step:end', function (step) {
            var allure = this.getAllure(step.cid);
            logger('step:end', step);

            if (!this.isAnyTestRunning(allure)) {
                error('ERROR', 'cannot end step because no test is running', step);
                return;
            }

            if (step.arg) {
                this.dumpJSON(allure, 'Step Result', step.arg);
            }

            if (step.assertionFailures) {
                this.dumpJSON(allure, 'Verification Failures', step.assertionFailures);
            }

            allure.endStep(step.status);
        });

        this.on('step:pass', function (step) {
            var allure = this.getAllure(step.cid);
            logger('step:pass', step);

            allure.endStep('passed');
        });

        this.on('step:fail', function (step) {
            var allure = this.getAllure(step.cid);
            logger('step:fail', step);

            this.dumpJSON(allure, 'Verification Failures', step.err);

            allure.endStep('failed');
        });

        this.on('step:unverified', function (step) {
            var allure = this.getAllure(step.cid);
            logger('step:unverified', step);

            this.dumpJSON(allure, 'Unverified Specifications', step.err);

            allure.endStep('unknown');
        });

        /*
          attachment : {
            cid: ...,
            event: 'test:attach',
            title: string,
            file: string,
            type: string
          }
         */
        this.on('test:attach', function (attachment) {
            var allure = this.getAllure(attachment.cid);
            logger('test:attach', attachment);

            if (this.isAnyTestRunning(allure)) {
                error('ERROR', 'cannot attach because no test is running', attachment);
                return;
            }
            allure.addAttachment(attachment.title, fs.readFileSync(attachment.file), attachment.type);
        });

        /*
          log : {
            cid: ...,
            event: 'test:log',
            message: string,
            detail: object
          }
         */
        this.on('test:log', function (log) {
            var allure = this.getAllure(log.cid);
            logger('test:log', log);

            if (this.isAnyTestRunning(allure)) {
                error('ERROR', 'cannot log because no test is running', log);
                return;
            }
            var content = log.detail != null ? JSON.stringify(log.detail, null, '    ') : '';
            allure.addAttachment(log.message, content, 'application/json');
        });

        this.on('runner:screenshot', function (command) {
            var allure = this.getAllure(command.cid);
            allure.addAttachment('screenshot ' + command.filename, new Buffer(command.data, 'base64'));
        });

        this.on('hook:start', function (hook) {
            var allure = _this.getAllure(hook.cid);

            if (!allure.getCurrentSuite() || LOGGING_HOOKS.indexOf(hook.title) === -1) {
                return;
            }

            allure.startCase(hook.title);
        });

        this.on('hook:end', function (hook) {
            var allure = _this.getAllure(hook.cid);

            if (!allure.getCurrentSuite() || LOGGING_HOOKS.indexOf(hook.title) === -1) {
                return;
            }

            allure.endCase('passed');

            if (allure.getCurrentTest().steps.length === 0) {
                allure.getCurrentSuite().testcases.pop();
            }
        });
    }

    _createClass(AllureReporter, [{
        key: 'handleMetadata',
        value: function handleMetadata(meta) {
            var allure = this.getAllure(meta.cid);
            logger('test:meta', meta);

            if (!this.isAnyTestRunning(allure)) {
                error('ERROR', 'test:meta : NO TEST RUNNING');
                return;
            }
            var currentTest = allure.getCurrentTest();

            // manage description
            if (meta.description) {
                currentTest.setDescription(meta.description);
            }
            // manage labels ( feature, story, issue )
            if (meta.feature) {
                if (typeof meta.feature === 'string') {
                    currentTest.addLabel('feature', meta.feature);
                } else {
                    for (var i in meta.feature) {
                        currentTest.addLabel('feature', meta.feature[i]);
                    }
                }
            }
            if (meta.story) {
                if (typeof meta.story === 'string') {
                    currentTest.addLabel('story', meta.story);
                } else {
                    for (var i in meta.story) {
                        currentTest.addLabel('story', meta.story[i]);
                    }
                }
            }
            if (meta.issue) {
                if (typeof meta.issue === 'string') {
                    currentTest.addLabel('issue', meta.issue);
                } else {
                    for (var i in meta.issue) {
                        currentTest.addLabel('issue', meta.issue[i]);
                    }
                }
            }
            if (meta.severity) {
                if (typeof meta.severity === 'string') {
                    currentTest.addLabel('severity', meta.severity);
                } else {
                    error('ERROR : meta.severity should be a string', meta);
                }
            }
            // manage parameters
            if (meta.argument) {
                if (typeof meta.argument === 'object') {
                    var keys = _Object$keys(meta.argument);
                    for (var i in keys) {
                        var key = keys[i];
                        var val = meta.argument[key];
                        currentTest.addParameter('argument', key, val);
                    }
                } else {
                    error('ERROR : meta.argument should be an object { name2: val1, name2: val2.. }', meta);
                }
            }
            if (meta.environment) {
                if (typeof meta.environment === 'object') {
                    var keys = _Object$keys(meta.environment);
                    for (var i in keys) {
                        var key = keys[i];
                        var val = meta.environment[key];
                        currentTest.addParameter('environment-variable', key, val);
                    }
                } else {
                    error('ERROR : meta.environment should be an object { name2: val1, name2: val2.. }', meta);
                }
            }
        }

        // add test.arguments to current test
    }, {
        key: 'addArguments',
        value: function addArguments(test) {
            if (test.arguments) {
                for (var key in test.arguments) {
                    if (test.arguments[key].value) {
                        this.getAllure(test.cid).getCurrentTest().addParameter('argument', test.arguments[key].caption, test.arguments[key].value);
                    }
                }
            }
        }
    }, {
        key: 'getAllure',
        value: function getAllure(cid) {
            if (typeof cid === 'undefined') {
                cid = process.workflo.cid;
            } else {
                if (typeof process.workflo === 'undefined') {
                    process.workflo = {};
                } else {
                    process.workflo.currentCid = cid; // REMOVE???
                }
            }

            if (this.allures[cid]) {
                return this.allures[cid];
            }

            var allure = new _allureJsCommonsWorkflo2['default']();
            allure.setOptions({ targetDir: this.options.outputDir || 'allure-results' });
            this.allures[cid] = allure;
            return this.allures[cid];
        }
    }, {
        key: 'isAnyTestRunning',
        value: function isAnyTestRunning(allure) {
            return allure.getCurrentSuite() && allure.getCurrentTest();
        }
    }, {
        key: 'dumpJSON',
        value: function dumpJSON(allure, name, json) {
            var jsonStr = JSON.stringify(json, null, '    ').split('\\"').join('\'').replace(/(?:\\n)/g, '');
            allure.addAttachment(name, jsonStr, 'application/json');
        }
    }]);

    return AllureReporter;
})(_events2['default'].EventEmitter);

exports['default'] = AllureReporter;
module.exports = exports['default'];
